# EMBEDDED LINUX

# Firmware deployment onto the SD card

**Brief**
From the first chapter, we built a full Debian distribution that is now ready to be sent to the SD card. From the second chapter, we formatted the SD card so it's now ready to host a filesystem. You got it: this chapter's about copy-pasting our Debian files onto the SD card. That's what the deployment is all about.

**Author** 
[dimitri.boudier@ensicaen.fr](mailto:dimitri.boudier@ensicaen.fr)
[hugo.descoubes@ensicaen.fr](mailto:hugo.descoubes@ensicaen.fr)

**Source**
[Debian: Getting started with the BeagleBone Black.](https://forum.digikey.com/t/debian-getting-started-with-the-beaglebone-black/12967)
by Robert C. Nelson

**Reminder**
/!\ Understand all commands before running them! /!\



-------------------------------------------------------------------------------
## RUN THE START-UP SCRIPT


Run the `elinux/tp/disco/script/start_elinux.sh` script that moves you to your working directory

```shell
source script/start_elinux.sh
```



-------------------------------------------------------------------------------
## DEPLOY ALL IMAGES ON MMC SD CARD


### BOOTLOADER

You have already built the 2nd-stage and 3rd-stage bootloaders in the first chapter. They are stored in the `${DISCOPATH}/deploy/` directory.

```shell
cd ${DISCOPATH}
```



#### Install MLO - Texas Instrument 2nd boot level

Copy the MLO (2nd-stage bootloader) onto the SD card.

```shell
sudo dd if=./deploy/MLO of=${DISK} count=1 seek=1 bs=128k
```

Let's analyse what has been written onto the card.

```shell
sudo dd if=${DISK} of=./mass/sdcard_dump_step2_mlo bs=1 count=1M
xxd mass/sdcard_dump_step2_mlo > mass/sdcard_dump_step2_mlo.txt
gedit mass/sdcard_dump_step2_mlo.txt &
```
* What is the start address of the MLO on the SD device? Does it match the first `dd` command?
> 
> 
> 
> 

* How does the 1st-stage bootloader (i.e. Boot Rom, the TI firmware) know where the MLO is stored? Does the start address really matter? See [here](https://groups.google.com/g/beagleboard/c/8fe5054qZss?pli=1)
> 
> 
> 
> 



#### Install U-Boot - 3rd boot level

Copy the U-Boot (3rd-stage bootloader) onto the SD card.

```shell
sudo dd if=./deploy/u-boot.img of=${DISK} count=2 seek=1 bs=384k
```


Let's analyse what has been written onto the card.

```shell
sudo dd if=${DISK} of=./mass/sdcard_dump_step3_uboot bs=1 count=1M
xxd mass/sdcard_dump_step3_uboot > mass/sdcard_dump_step3_uboot.txt
gedit mass/sdcard_dump_step3_uboot.txt &
```
* What is the start address of the U-boot on the SD device? Does it match the first `dd` command?
> 
> 
> 
> 



### KERNEL

#### Install kernel and dtb on microSD card

You have already built the kernel image and the device tree binary in the first chapter. They are stored in the `${DISCOPATH}/deploy/` directory. Also, check if your partition is correctly mounted. If not, mount it.

```shell
cd ${DISCOPATH}/deploy
lsblk -f
```

Copy the files onto the mounted partition.

```shell
sudo mkdir -p ${MEDIA}/rootfs/boot/

export kernel_version=4.14.198-bone-rt-r40
sudo sh -c "echo 'uname_r=${kernel_version}' >> ${MEDIA}/rootfs/boot/uEnv.txt"

sudo cp -v ./zImage ${MEDIA}/rootfs/boot/vmlinuz-${kernel_version}
sudo cp -v ./am335x-boneblack.dtb ${MEDIA}/rootfs/boot/

```
* What files have been written onto the SD card partition?
> 
> 
> 
> 



### ROOTFS

#### Install Debian rootfs on microSD card

You have already downloaded an archive of the Debian filesystem in the first chapter. It is stored in the `${DISCOPATH}/rootfs/debian/` directory.

```shell
cd ${DISCOPATH}/rootfs/debian
sudo tar xfvp ./*-*-*-armhf-*/armhf-rootfs-*.tar -C ${MEDIA}/rootfs/

sudo chown root:root ${MEDIA}/rootfs/
sudo chmod 755 ${MEDIA}/rootfs/
```
* Check at the content of your SD card partition. What is there now?
> 
> 
> 
> 



-------------------------------------------------------------------------------
## SYNC THE PARTITION

Sync, but wait for the operation to be complete (approx 1-2 min).

```shell
sync
sudo umount ${MEDIA}/rootfs
```

Verify that your mounting point `${MEDIA}/rootfs/` is correctly unmounted.

```shell
lsblk -f
```



-------------------------------------------------------------------------------
## VERIFY BBB BOOT SEQUENCE

Now that a full distribution is on the SD card, we will check if the BeagleBone Black would start with your SD card.



### Prepare Minicom

1. Plug the USB/Serial TTL cable into your computer
2. Check for the corresponding device (`ls /dev`)
3. Open minicom (`sudo minicom -D /dev/*your_device*`)
4. Configure Minicom (`Ctrl+A`, then `Z`, then `O`)
  + 115200 bauds, 8 bit, 1 stop bit, no parity, no flux control



### Prepare the BeagleBoneBlack

5. Plug the SD card into the BBB
6. Connect the USB/Serial TTL cable to the BBB
  + the black wire of the USB/Serial TTL cable is the ground connection
  + connector J1 on BBB, the dot next to the connector indicates the ground



### Start the BeagleBone Black and see

7. Hold down the USER/BOOT button (marked S2)
8. Apply power
9. When all blue lights will turn on, you can release the USER/BOOT button

In the meantime, you should have seen many messages on the Minicom interface. We will analyse them later on.



-------------------------------------------------------------------------------
## UPDATE THE start_elinux.sh SCRIPT

Update the `elinux/tp/disco/script/start_elinux.sh` script by adding the definition of the variable `kernel_version`.



-------------------------------------------------------------------------------

## CREATE A SCRIPT FOR SETTING UP THE SD CARD

Create a new script `elinux/tp/disco/script/sdcard_deploy.sh`. It should contain only the necessary commands deploy the Bootload / Kernel-image / Device-tree / Debian-fs onto the SD Card.

* Copy here the content of your script.
> 
> 
>